@@ -3,20 +3,33 @@ from __future__ import division, with_statement
 
 """Segtools: tools for exploratory analysis of genomic segmentations
 
+Copyright 2011: Michael Hoffman <mmh1@uw.edu>
 Copyright 2009: Orion Buske <stasis@uw.edu>
-
 """
-__version__ = "$Revision: 273 $"
+__version__ = "$Revision: 322 $"
 
 import os
 import re
 import sys
 
 from collections import defaultdict
+from contextlib import contextmanager
 from functools import partial
 from numpy import array, int64, uint32
+from time import time
 
-PICKLED_EXT = "pkl"
+from pkg_resources import resource_filename
+
+try:
+    PKG = __package__  # Python 2.6
+except NameError:
+    PKG = "segtools"
+
+PKG_R = os.extsep.join([PKG, "R"])
+
+EXT_GZ = "gz"
+EXT_PICKLE = "pkl"
+EXT_R = "R"
 
 DTYPE_SEGMENT_START = int64
 DTYPE_SEGMENT_END = int64
@@ -28,7 +41,8 @@ class Annotations(object):
 
     chromosomes: a dict
       key: chromosome name
-      val: segments, a numpy.ndarray, (each row is a (start, end, key) struct)
+      val: segments, a numpy.ndarray,
+           (each row is a (start, end, key, [strand, ...]) struct)
            sorted by start, end
            * These segments are not necessarily non-overlapping
     labels: a dict
@@ -61,8 +75,11 @@ class Annotations(object):
         if not os.path.isfile(filename):
             raise IOError("Could not find file: %s" % filename)
 
-        log("Loading %s:" % self.__class__, verbose)
+        log("Loading %s from file: %s" %
+            (self.__class__.__name__, filename), verbose)
+        start = time()
         self._load(filename, verbose=verbose)
+        log("Loading finished in %.1f seconds." % (time() - start), verbose)
 
     @staticmethod
     def _get_file_format(filepath):
@@ -83,13 +100,13 @@ class Annotations(object):
 
     def _load(self, filename, verbose=True):
         format = self._get_file_format(filename)
-        if format == PICKLED_EXT:
+        if format == EXT_PICKLE:
             # Read pickled object file
             self._from_pickle(filename, verbose=verbose)
         elif format in set(["bed", "gff", "gtf"]):
             self._from_file(filename, verbose=verbose)
         else:
-            raise self.FormatError("Unrecognized file format: %s on file: %s"
+            raise self.FormatError("Unrecognized extension (%s) on file: %s"
                                    % (format, filename))
 
     def _iter_rows(self, filename, verbose=True):
@@ -109,6 +126,8 @@ class Annotations(object):
             raise self.FormatError("Cannot iterate segments in file format:"
                                    " %s" % format)
 
+        log("  Parsing lines from %s format" % format, verbose)
+
         with maybe_gzip_open(filename) as infile:
             for datum in reader(infile):
                 row = {}
@@ -135,18 +154,20 @@ class Annotations(object):
 
     def _from_pickle(self, filename, verbose=True):
         import cPickle
+        from .common import maybe_gzip_open
 
-        log("  Unpickling %s object..." % self.__class__,
-            verbose, end="")
+        log("  Unpickling %s" % self.__class__.__name__, verbose)
 
-        with open(filename, 'rb') as ifp:
+        with maybe_gzip_open(filename, 'rb') as ifp:
             object = cPickle.load(ifp)
-            if object.__class__ != self.__class__:
-                raise self.UnpickleError("Error: Trying to load an indexed %s"
-                                         " object as an indexed %s object!")
-            self.__dict__ = object.__dict__
+            if not issubclass(object.__class__, self.__class__):
+                msg = ("Error: Cannot to load an indexed %s object"
+                       " as an indexed %s object."
+                       % (object.__class__.__name__,
+                          self.__class__.__name__))
+                raise self.UnpickleError(msg)
 
-        log(" done", verbose)
+            self.__dict__ = object.__dict__
 
     def _from_file(self, filename, verbose=True):
         """Parses a data file and sets object attributes
@@ -219,16 +240,12 @@ class Annotations(object):
 
         # Sort segments within each chromosome
         if unsorted_chroms:
-            log("  Segments were unsorted relative to the following"
+            log("  Sorting unsorted segments in the following"
                 " chromosomes: %s" % ", ".join(unsorted_chroms), verbose)
-            log("  Sorting...", verbose, end="")
-
             for chrom in unsorted_chroms:
                 segments = chromosomes[chrom]
                 segments.sort(order=['start'])
 
-            log(" done", verbose)
-
         self.filename = filename
         self.chromosomes = chromosomes
         self._labels = labels
@@ -236,21 +253,22 @@ class Annotations(object):
         self._n_label_bases = n_label_bases
         self._inv_labels = label_dict
 
-    def pickle(self, verbose=True, clobber=False):
+    def pickle(self, filename=None, verbose=True, clobber=False):
         """Pickle the annotations into an output file"""
         import cPickle
-        from .common import check_clobber
+        from .common import check_clobber, maybe_gzip_open
 
-        filename = os.extsep.join([self.filename, PICKLED_EXT])
+        if filename is None:
+            filename = self.filename
+
+        filename = os.extsep.join([filename, EXT_PICKLE, EXT_GZ])
 
         check_clobber(filename, clobber)
-        log("Pickling %s object to file: %s..." % (self.__class__, filename),
-            verbose, end="")
-        with open(filename, 'wb') as ofp:
+        log("Pickling %s object to file: %s"
+            % (self.__class__.__name__, filename), verbose)
+        with maybe_gzip_open(filename, 'wb') as ofp:
             cPickle.dump(self, ofp, -1)
 
-        log(" done", verbose)
-
     def num_label_segments(self, label):
         return self._n_label_segments[str(label)]
 
@@ -301,8 +319,8 @@ class Segmentation(Annotations):
 
     def _iter_rows(self, filename, verbose=True):
         """Override default row reading to ignore strand for Segmentations"""
-        for row in super(self.__class__, self)._iter_rows(filename,
-                                                          verbose=verbose):
+        for row in super(self.__class__,
+                         self)._iter_rows(filename, verbose=verbose):
             row['strand'] = '.';
             yield row;
 
@@ -315,7 +333,6 @@ class Segmentation(Annotations):
         """
         super(self.__class__, self)._from_file(filename, verbose=verbose)
 
-        log("  Checking for overlapping segments...", verbose, end="")
         for chrom, segments in self.chromosomes.iteritems():
             # Make sure there are no overlapping segments
             if segments.shape[0] > 1 and \
@@ -323,12 +340,12 @@ class Segmentation(Annotations):
                 raise self.SegmentOverlapError("Found overlapping segments"
                                                " in chromosome: %s" % chrom)
 
-        log(" done", verbose)
         self.segtool, self.tracks = self.get_bed_metadata(filename)
 
     @staticmethod
     def get_bed_metadata(filename):
         from .common import maybe_gzip_open
+
         regexp = re.compile('description="(.*) segmentation of (.*)"')
 
         segtool = ""
@@ -356,6 +373,144 @@ def log(message, verbose=True, end="\n", file=sys.stderr):
         file.write(str(message))
         file.write(end)
 
+## Die with error message
+def die(msg="Unexpected error"):
+    log("\nFatal error: %s\n" % msg)
+    sys.exit(1)
+
+
+## Class to handle sourcing, plotting, and calling R functions
+class RInterface(object):
+    """
+    Interface to R environment, providing methods to source files,
+    call functions, and plot with R
+    """
+    _get_filename = partial(resource_filename, PKG_R)
+    def __init__(self, files_to_source=[], verbose=True):
+        self.started = False
+        self._files = tuple(files_to_source)
+        self.verbose = verbose
+        self._r_console = lambda msg: sys.stderr.write(str(msg))
+
+    def start(self, transcriptfile=None, verbose=None):
+        if verbose is not None:
+            self.verbose = verbose
+
+        if self.started:
+            return
+
+        self.started = True
+
+        # Start up R
+        log("Opening connection to R environment.", self.verbose)
+        from rpy2.robjects import r, rinterface
+        self._r = r
+        self._rinterface = rinterface
+        self.RError = rinterface.RRuntimeError
+
+        # Set up transcript
+        self._transcript = transcriptfile
+        if self._transcript:
+            print >>self._transcript
+            log("Saving transcript to file: %s" % self._transcript.name,
+                self.verbose)
+
+        # Source any R files
+        for file in self._files:
+            self.source(file)
+
+    def source(self, filename):
+        """Simplify importing R source in the package"""
+        self.start()
+
+        filename_full = self._get_filename(filename)
+        if self._transcript:
+            print >>self._transcript, "source(%r)" % filename_full
+
+        try:
+            self._r.source(filename_full)
+        except self.RError:
+            die("Failed to load R package: %s\n" % filename_full)
+
+    @classmethod
+    def arg_to_text(cls, arg):
+        if isinstance(arg, bool):
+            return repr(arg).upper() # True -> TRUE, False -> FALSE
+        else:
+            return repr(arg)
+
+    def call(self, func, *args, **kwargs):
+        """Safer way to call R functions (without importing all that junk)
+
+        None values are substituted with empty string
+        """
+        self.start()
+
+        from rpy2.robjects import numpy2ri
+        # numpy2ri imported for side-effect
+
+        # Make sure there are no None values in args or kwargs
+        # rpy2 currently doesn't support passing Nones, so replace them with ""
+        for arg_i, arg in enumerate(args):
+            if arg is None:
+                args[arg_i] = ""
+
+        for key, val in kwargs.iteritems():
+            if val is None:
+                kwargs[key] = ""
+
+        # Save call to transcript
+        if self._transcript:
+            args_text = [self.arg_to_text(arg) for arg in args]
+            kwargs_text = ["%s = %s" % (key, self.arg_to_text(value))
+                           for key, value in kwargs.iteritems()]
+            all_args = ", ".join(args_text + kwargs_text)
+
+            print >>self._transcript, "%s(%s)" % (func, all_args)
+
+        return self._r[func](*args, **kwargs)
+
+    def plot(self, func, *args, **kwargs):
+        """Call R function but print timing information
+
+        (kept around for backwards compatibility)
+        """
+        self.start()
+        log("Plotting with R function: %r" % func, self.verbose)
+        start = time()
+
+        # Unless verbose, only print R output when there is an error
+        r_log = []
+        if self.verbose:
+            r_console = self._r_console
+        else:
+            r_console = lambda msg: r_log.append(str(msg))
+
+        self._rinterface.set_writeconsole(r_console)
+        try:
+            result = self.call(func, *args, **kwargs)
+        except self.RError:
+            die("Encoundered error within R:\n%s" % "\n  ".join(r_log))
+
+        # Return console back to stderr
+        self._rinterface.set_writeconsole(self._r_console)
+
+        log("Plotting completed in %.1f seconds" % (time() - start),
+            self.verbose)
+        return result
+
+
+@contextmanager
+def open_transcript(dirpath, module, verified=False):
+    filename = os.path.join(dirpath, os.extsep.join([module, EXT_R]))
+    with open(filename, "w") as res:
+        if not verified:
+            print >>res, "## Experimental R transcript"
+            print >>res, "## You may not be able to run the R code in this file exactly as written."
+            print >>res
+
+        yield res
+
 if __name__ == "__main__":
     pass
 
