@@ -14,7 +14,7 @@ as provided by exporting UCSC gene data in GTF format.
 # A package-unique, descriptive module name used for filenames, etc
 MODULE="feature_aggregation"
 
-__version__ = "$Revision: 292 $"
+__version__ = "$Revision: 321 $"
 
 import os
 import sys
@@ -24,10 +24,10 @@ from functools import partial
 from numpy import arange, array, empty, linspace, round
 
 from . import (Annotations, log, Segmentation, DTYPE_SEGMENT_START,
-               DTYPE_SEGMENT_END, DTYPE_SEGMENT_KEY, DTYPE_STRAND)
-from .common import (die, fill_array, get_ordered_labels, make_tabfilename,
-                     map_segment_label, r_plot, r_source, setup_directory,
-                     tab_saver)
+               DTYPE_SEGMENT_END, DTYPE_SEGMENT_KEY, DTYPE_STRAND,
+               die, RInterface)
+from .common import (fill_array, get_ordered_labels, make_tabfilename,
+                     map_segment_label, setup_directory, tab_saver)
 from .html import save_html_div
 
 NAMEBASE = "%s" % MODULE
@@ -92,6 +92,8 @@ REGION_BINS = 50
 INTRON_BINS = 50
 EXON_BINS = 25
 
+R = RInterface(["common.R", "aggregation.R"])
+
 class GeneAnnotations(Annotations):
     """Annotations class for gene-type annotations"""
 
@@ -423,10 +425,6 @@ class GeneAnnotations(Annotations):
         return res
 
 
-def start_R():
-    r_source("common.R")
-    r_source("aggregation.R")
-
 def print_bed_from_gene_component(features, component="terminal exon"):
     with open("%s.bed" % component, "w") as ofp:
         for chrom, chrom_features in features.iteritems():
@@ -735,7 +733,7 @@ def save_tab(segmentation, features, counts, components, component_bins,
 ## Plots aggregation data from tab file
 def save_plot(dirpath, mode, namebase=NAMEBASE, clobber=False, verbose=True,
               mnemonic_file=None, normalize=False, significance=False):
-    start_R()
+    R.start(verbose=verbose)
 
     tabfilename = make_tabfilename(dirpath, namebase)
     if not os.path.isfile(tabfilename):
@@ -746,16 +744,16 @@ def save_plot(dirpath, mode, namebase=NAMEBASE, clobber=False, verbose=True,
 
     # Plot data in tab file
     if mode == GENE_MODE:
-        r_plot("save.gene.aggregations", dirpath, NAMEBASE_SPLICING,
+        R.plot("save.gene.aggregations", dirpath, NAMEBASE_SPLICING,
                NAMEBASE_TRANSLATION, tabfilename, mnemonic_file=mnemonic_file,
-               normalize=normalize, clobber=clobber, verbose=verbose,
-               significance=significance)
+               normalize=normalize, clobber=clobber, significance=significance)
     else:
-        r_plot("save.aggregation", dirpath, namebase, tabfilename,
+        R.plot("save.aggregation", dirpath, namebase, tabfilename,
                mnemonic_file=mnemonic_file, normalize=normalize,
-               clobber=clobber, verbose=verbose, significance=significance)
+               clobber=clobber, significance=significance)
 
-def save_html(dirpath, featurefilename, mode, clobber=False, normalize=False):
+def save_html(dirpath, featurefilename, mode, clobber=False,
+              verbose=True, normalize=False):
     featurebasename = os.path.basename(featurefilename)
     title = "%s (%s)" % (HTML_TITLE_BASE, featurebasename)
     if normalize:
@@ -773,7 +771,8 @@ def save_html(dirpath, featurefilename, mode, clobber=False, normalize=False):
 
     save_html_div(templatefile, dirpath, NAMEBASE, clobber=clobber,
                   module=MODULE, featurefilename=featurebasename, mode=mode,
-                  title=title, yaxis=yaxis, extra_namebases=extra_namebases)
+                  title=title, yaxis=yaxis, verbose=verbose,
+                  extra_namebases=extra_namebases)
 
 def print_array(arr, tag="", type="%d"):
     if len(arr) > 2:
@@ -870,7 +869,7 @@ def validate(bedfilename, featurefilename, dirpath,
                   mnemonic_file=mnemonic_file, normalize=normalize,
                   significance=significance)
 
-    save_html(dirpath, featurefilename, mode=mode,
+    save_html(dirpath, featurefilename, mode=mode, verbose=verbose,
               clobber=clobber, normalize=normalize)
 
 def parse_options(args):
@@ -885,7 +884,7 @@ def parse_options(args):
     parser = OptionParser(usage=usage, version=version,
                           description=description)
 
-    group = OptionGroup(parser, "Input options")
+    group = OptionGroup(parser, "Output options")
     group.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
                       default=None, metavar="FILE",
                       help="If specified, labels will be shown using"
@@ -926,15 +925,15 @@ def parse_options(args):
                      " normalized by the number of segments in that group"
                      " instead of the raw counts"
                      " (normalize over SEGMENTATION labels)")
-    group.add_option("--significance", action="store_true",
-                     dest="significance", default=False,
-                     help="Include the significance of the aggregation"
-                     " at each base in the plot. Warning: significance"
-                     " is approximated using a binomial distribution"
-                     " adjusted to a q-value threshold. Uses the QVALUE"
-                     " package which must be installed. This method does"
-                     " not always yield reasonable results, so use with"
-                     " extreme caution.")
+#     group.add_option("--significance", action="store_true",
+#                      dest="significance", default=False,
+#                      help="Include the significance of the aggregation"
+#                      " at each base in the plot. Warning: significance"
+#                      " is approximated using a binomial distribution"
+#                      " adjusted to a q-value threshold. Uses the QVALUE"
+#                      " package which must be installed. This method does"
+#                      " not always yield reasonable results, so use with"
+#                      " extreme caution.")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Main aggregation options")
@@ -991,7 +990,7 @@ def main(args=sys.argv[1:]):
               "by_groups": options.by_groups,
               "normalize": options.normalize,
               "mode": options.mode,
-              "significance": options.significance,
+#               "significance": options.significance,
               "mnemonic_file": options.mnemonic_file}
     validate(bedfilename, featurefilename, options.outdir, **kwargs)
 
