@@ -1,6 +1,6 @@
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 295 $"
+__version__ = "$Revision: 320 $"
 
 """
 Assorted utility functions and classes common or useful to most of Segtools.
@@ -9,6 +9,8 @@ Assorted utility functions and classes common or useful to most of Segtools.
 Author: Orion Buske <stasis@uw.edu>
 """
 
+# XXX: the things in here should be moved to __init__; test
+
 import os
 import sys
 
@@ -16,23 +18,15 @@ from contextlib import closing, contextmanager
 from functools import partial
 from gzip import open as _gzip_open
 from warnings import warn
+from time import time
 
 from numpy import array, empty, iinfo, ndarray, searchsorted
-from pkg_resources import resource_filename
 from tabdelim import DictReader, DictWriter, ListReader, ListWriter
-from rpy2.robjects import r, rinterface
 
-from . import log
+from . import log, EXT_GZ, PKG, die
 
-try:
-    PKG = __package__  # Python 2.6
-except NameError:
-    PKG = "segtools"
-
-PKG_R = os.extsep.join([PKG, "R"])
 PKG_RESOURCE = os.extsep.join([PKG, "resources"])
 
-EXT_GZ = "gz"
 EXT_PDF = "pdf"
 EXT_PNG = "png"
 EXT_TAB = "tab"
@@ -54,9 +48,6 @@ LABEL_ALL = "all"
 
 THUMB_SIZE = 100
 
-r_filename = partial(resource_filename, PKG_R)
-RError = rinterface.RRuntimeError
-
 class OutputMasker(object):
     """Class to mask the output of a stream.
 
@@ -241,11 +232,6 @@ class FeatureScanner(object):
 
 ## UTILITY FUNCTIONS
 
-## Die with error message
-def die(msg="Unexpected error"):
-    log("\nERROR: %s\n" % msg)
-    sys.exit(1)
-
 def inverse_dict(d):
     """Given a dict, returns the inverse of the dict (val -> key)"""
     res = {}
@@ -295,51 +281,6 @@ def fill_array(scalar, shape, dtype=None, *args, **kwargs):
     res.fill(scalar)
     return res
 
-def r_source(filename):
-    """Simplify importing R source in the package"""
-
-    try:
-        r.source(r_filename(filename))
-    except RError:
-        die("Failed to load R package: %s\n" % filename)
-
-def r_call(func, *args, **kwargs):
-    """Safer way to call R functions (without importing all that junk)
-
-    None values are substituted with ""
-    """
-    from rpy2.robjects import numpy2ri
-    # numpy2ri imported for side-effect
-
-    # Make sure there are no None values in args or kwargs
-    # rpy2 currently doesn't support passing Nones, so replace them with ""
-    for arg_i, arg in enumerate(args):
-        if arg is None:
-            args[arg_i] = ""
-
-    for key, val in kwargs.iteritems():
-        if val is None:
-            kwargs[key] = ""
-
-    return r[func](*args, **kwargs)
-
-def r_plot(func, *args, **kwargs):
-    """Safely call R plotting function (with verbose print option)
-
-    Calls r_call
-    Argument "verbose" is caught by r_plot and not passed on to R functions
-    """
-
-    # Grab verbose value, with default to True
-    verbose = kwargs.pop("verbose", True)
-
-    log("Saving plots with R function: %r..." % func, verbose, end="")
-
-    result = r_call(func, *args, **kwargs)
-    log(" done", verbose)
-
-    return result
-
 @contextmanager
 def tab_saver(dirpath, basename, fieldnames=None, header=True,
               clobber=False, metadata=None, verbose=True):
@@ -352,9 +293,9 @@ def tab_saver(dirpath, basename, fieldnames=None, header=True,
       Comment line will start with '# ' and then will be a space-delimited
       list of <field>=<value> pairs, one for each element of the dict.
     """
-    log("Saving tab file...", verbose, end="")
-
     outfilename = make_tabfilename(dirpath, basename)
+    log("Saving tab file: %s" % outfilename, verbose)
+    start = time()
     check_clobber(outfilename, clobber)
     with open(outfilename, "w") as outfile:
         if metadata is not None:
@@ -368,7 +309,7 @@ def tab_saver(dirpath, basename, fieldnames=None, header=True,
         else:
             yield ListWriter(outfile)
 
-    log(" done", verbose)
+    log("Saved tab file in %.1f seconds" % (time() - start), verbose)
 
 @contextmanager
 def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
@@ -380,9 +321,10 @@ def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
 
     metadata: a dict describing the data included in the comment line.
     """
-    log("Reading tab file...", verbose, end="")
 
     infilename = make_tabfilename(dirpath, basename)
+    log("Reading tab file: %s" % infilename, verbose)
+    start = time()
     if not os.path.isfile(infilename):
         raise IOError("Unable to find tab file: %s" % infilename)
 
@@ -403,7 +345,7 @@ def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
         else:
             yield ListReader(infile), metadata
 
-    log(" done", verbose)
+    log("Tab file read in %.1f seconds" % (time() - start), verbose)
 
 def map_segment_label(segments, range):
     """Flatten segments to a segment label per base vector
@@ -532,8 +474,7 @@ def setup_directory(dirname):
     else:
         # Require write and execute permissions on existing dir
         if not os.access(dirname, os.W_OK | os.X_OK):
-            die("Error: Output directory does not have appropriate"
-                " permissions.")
+            die("Error: Output directory is not writeable and executable.")
 
 ## Given labels and mnemonics, returns an ordered list of label_keys
 ##   and a new labels dict mapping label_keys to label strings
@@ -575,13 +516,13 @@ def map_mnemonics(labels, mnemonicfilename, field="new"):
     if not mnemonicfilename:
         return []
 
-    label_order, label_data = load_mnemonics(mnemonicfilename)
+    label_order, label_mnemonics = load_mnemonics(mnemonicfilename)
     str_labels = labels.values()
     mnemonics = []
     # Add mapping for labels in mnemonic file
     for old_label in label_order:
         try:
-            new_label = label_data[old_label][field]
+            new_label = label_mnemonics[old_label][field]
         except KeyError:
             die("Mnemonic file missing expected column: %s" % field)
 
@@ -598,7 +539,7 @@ def map_mnemonics(labels, mnemonicfilename, field="new"):
     return array(mnemonics)
 
 def make_comment_ignoring_dictreader(iterable, *args, **kwargs):
-    return DictReader(item for item in iterable if not item.startswith("#"),
+    return DictReader((item for item in iterable if not item.startswith("#")),
                       *args, **kwargs)
 
 ## Loads segmentation label descriptions and mnemonics
@@ -608,7 +549,7 @@ def load_mnemonics(filename):
     Input file should have a tab-delimited row for each label, of the form:
                old    new    description
       e.g.     4    IS   initiation (strong)
-    Returns a tuple of (label_order, label_data)
+    Returns a tuple of (label_order, label_mnemonics)
 
     label_order: an ordered list of old labels,
       corresponding to the preferred display order in plots
@@ -625,7 +566,7 @@ def load_mnemonics(filename):
         die("Could not find mnemonic file: %s" % filename)
 
     label_order = []
-    label_data = {}
+    label_mnemonics = {}
     with open(filename, "rU") as ifp:
         reader = make_comment_ignoring_dictreader(ifp)
         for row in reader:
@@ -635,6 +576,6 @@ def load_mnemonics(filename):
                 die("Mnemonic file missing required column: 'old'")
 
             label_order.append(label_index)
-            label_data[label_index] = row
+            label_mnemonics[label_index] = row
 
-    return (label_order, label_data)
+    return (label_order, label_mnemonics)
