@@ -2,8 +2,8 @@
 from __future__ import division, with_statement
 
 """
-Provides command-line and package entry points for analyzing the segment
-length distribution in a provided BED-formatted segmentation.
+Provides command-line and package entry points for analyzing the
+length distribution of entries in a provided segmentation or annotations file.
 
 """
 
@@ -11,7 +11,7 @@ length distribution in a provided BED-formatted segmentation.
 # Must be the same as the folder containing this script
 MODULE="length_distribution"
 
-__version__ = "$Revision: 261 $"
+__version__ = "$Revision: 321 $"
 
 import os
 import sys
@@ -19,8 +19,9 @@ import sys
 from collections import defaultdict
 from numpy import concatenate, median
 
-from . import Segmentation
-from .common import die, get_ordered_labels, LABEL_ALL, make_tabfilename, r_plot, r_source, setup_directory, tab_saver
+from . import Annotations, die, RInterface, open_transcript
+from .common import get_ordered_labels, LABEL_ALL, make_tabfilename, \
+     setup_directory, tab_saver
 
 from .html import save_html_div
 
@@ -37,24 +38,23 @@ PNG_WIDTH = 600
 PNG_HEIGHT_BASE = 100  # Axes and label
 PNG_HEIGHT_PER_LABEL = 45
 
-def start_R():
-    r_source("common.R")
-    r_source("length.R")
+R = RInterface(["common.R", "length.R"])
 
-## Given a segmentation, returns the length of each segment
-def segmentation_lengths(segmentation):
+## Given annotations, returns the length of each annotation
+def annotations_lengths(annotations):
     # key: label_key
     # val: list of numpy.ndarray
     lengths = defaultdict(list)
-    labels = segmentation.labels
+    labels = annotations.labels
 
     # convert segment coords to lengths
-    for segments in segmentation.chromosomes.itervalues():
+    for rows in annotations.chromosomes.itervalues():
         for label_key in labels.iterkeys():
-            labeled_row_indexes = (segments['key'] == label_key)
-            labeled_rows = segments[labeled_row_indexes]
-            lengths[label_key].append(labeled_rows['end'] -
-                                      labeled_rows['start'])
+            labeled_row_indexes = (rows['key'] == label_key)
+            labeled_rows = rows[labeled_row_indexes]
+            length = labeled_rows['end'] - labeled_rows['start']
+            lengths[label_key].append(length)
+            lengths[LABEL_ALL].append(length)
 
     # key: label_key
     # val: int
@@ -87,10 +87,9 @@ def save_size_tab(lengths, labels, num_bp, dirpath, verbose=True,
     with tab_saver(dirpath, namebase, FIELDNAMES_SUMMARY,
                    clobber=clobber, verbose=verbose) as saver:
         # "all" row first
-        total_bp = sum(num_bp.itervalues())
-        all_lengths = concatenate(lengths.values())
-        row = make_size_row(LABEL_ALL, all_lengths,
-                               total_bp, total_bp)
+        total_bp = num_bp[LABEL_ALL]
+        row = make_size_row(LABEL_ALL, lengths[LABEL_ALL],
+                            total_bp, total_bp)
         saver.writerow(row)
 
         # Remaining label rows
@@ -127,64 +126,74 @@ def save_tab(lengths, labels, num_bp, dirpath, clobber=False, verbose=True):
 
 ## Generates and saves an R plot of the length distributions
 def save_plot(dirpath, namebase=NAMEBASE, clobber=False, verbose=True,
-              mnemonic_file=None):
-    start_R()
+              mnemonic_file=None, transcriptfile=None):
+    R.start(transcriptfile=transcriptfile, verbose=verbose)
 
     # Load data from corresponding tab file
     tabfilename = make_tabfilename(dirpath, namebase)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    r_plot("save.length", dirpath, namebase, tabfilename,
-           mnemonic_file=mnemonic_file, clobber=clobber, verbose=verbose)
+    R.plot("save.length", dirpath, namebase, tabfilename,
+           mnemonic_file=mnemonic_file, clobber=clobber)
 
 ## Generates and saves an R plot of the length distributions
 def save_size_plot(dirpath, namebase=NAMEBASE_SIZES, clobber=False,
-                   verbose=True, mnemonic_file=None):
-    start_R()
+                   verbose=True, mnemonic_file=None,
+                   show_bases=True, show_segments=True,
+                   transcriptfile=None):
+    R.start(transcriptfile=transcriptfile, verbose=verbose)
 
     # Load data from corresponding tab file
     tabfilename = make_tabfilename(dirpath, namebase)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    r_plot("save.segment.sizes", dirpath, namebase, tabfilename,
-           mnemonic_file=mnemonic_file, clobber=clobber, verbose=verbose)
+    R.plot("save.segment.sizes", dirpath, namebase, tabfilename,
+           mnemonic_file=mnemonic_file, clobber=clobber,
+           show_segments=show_segments, show_bases=show_bases)
 
-def save_html(dirpath, clobber=False, mnemonicfile=None):
+def save_html(dirpath, clobber=False, mnemonicfile=None, verbose=True):
     extra_namebases = {"sizes": NAMEBASE_SIZES}
     save_html_div(TEMPLATE_FILENAME, dirpath, namebase=NAMEBASE,
                   extra_namebases=extra_namebases, mnemonicfile=mnemonicfile,
                   tables={"": NAMEBASE_SIZES}, module=MODULE,
-                  clobber=clobber, title=HTML_TITLE)
+                  clobber=clobber, title=HTML_TITLE, verbose=verbose)
 
 ## Package entry point
-def validate(bedfilename, dirpath, clobber=False, replot=False, noplot=False,
-             verbose=True, mnemonic_file=None):
+def validate(filename, dirpath, clobber=False, replot=False, noplot=False,
+             verbose=True, mnemonic_file=None,
+             show_segments=True, show_bases=True):
     if not replot:
         setup_directory(dirpath)
-        segmentation = Segmentation(bedfilename, verbose=verbose)
+        annotations = Annotations(filename, verbose=verbose)
 
-        labels = segmentation.labels
+        labels = annotations.labels
 
-        lengths, num_bp=segmentation_lengths(segmentation)
+        lengths, num_bp = annotations_lengths(annotations)
         save_tab(lengths, labels, num_bp, dirpath,
                  clobber=clobber, verbose=verbose)
         save_size_tab(lengths, labels, num_bp, dirpath,
                       clobber=clobber, verbose=verbose)
 
     if not noplot:
-        save_plot(dirpath, mnemonic_file=mnemonic_file,
-                  clobber=clobber, verbose=verbose)
-        save_size_plot(dirpath, clobber=clobber, verbose=verbose,
-                       mnemonic_file=mnemonic_file)
+        with open_transcript(dirpath, MODULE) as transcriptfile:
+            save_plot(dirpath, mnemonic_file=mnemonic_file,
+                      clobber=clobber, verbose=verbose,
+                      transcriptfile=transcriptfile)
+            save_size_plot(dirpath, clobber=clobber, verbose=verbose,
+                           mnemonic_file=mnemonic_file,
+                           show_segments=show_segments,
+                           show_bases=show_bases,
+                           transcriptfile=transcriptfile)
 
-    save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file)
+    save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file,
+              verbose=verbose)
 
 def parse_options(args):
     from optparse import OptionGroup, OptionParser
 
-    usage = "%prog [OPTIONS] SEGMENTATION"
+    usage = "%prog [OPTIONS] ANNOTATIONS"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
 
@@ -203,6 +212,14 @@ def parse_options(args):
     group.add_option("--noplot", action="store_true",
                      dest="noplot", default=False,
                      help="Do not generate plots")
+    group.add_option("--no-segments", action="store_false",
+                     dest="show_segments", default=True,
+                     help="Do not show total segments covered by labels"
+                     " in size plot")
+    group.add_option("--no-bases", action="store_false",
+                     dest="show_bases", default=True,
+                     help="Do not show total bases covered by labels"
+                     " in size plot")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Output")
@@ -226,14 +243,11 @@ def parse_options(args):
 ## Command-line entry point
 def main(args=sys.argv[1:]):
     (options, args) = parse_options(args)
-    bedfilename = args[0]
+    filename = args[0]
 
-    kwargs = {"clobber": options.clobber,
-              "verbose": options.verbose,
-              "replot": options.replot,
-              "noplot": options.noplot,
-              "mnemonic_file": options.mnemonic_file}
-    validate(bedfilename, options.outdir, **kwargs)
+    kwargs = dict(options.__dict__)
+    outdir = kwargs.pop("outdir")
+    validate(filename, outdir, **kwargs)
 
 if __name__ == "__main__":
     sys.exit(main())
