@@ -1,24 +1,26 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 228 $"
+__version__ = "$Revision: 321 $"
 
 """
 Provides command-line and package entry points for analyzing nucleotide
-and dinucleotide frequencies for each segmentation label.
+and dinucleotide frequencies for each label in the segmentation or annotations
+file.
 
 """
 
 import numpy
 import os
 import sys
+import warnings
 
 from collections import defaultdict
 from genomedata import Genome
 from numpy import zeros
 
-from . import log, Segmentation
-from .common import die, make_tabfilename, r_plot,  r_source, setup_directory, tab_saver
+from . import log, Annotations, die, RInterface
+from .common import make_tabfilename, setup_directory, tab_saver
 
 from .html import save_html_div
 
@@ -48,13 +50,11 @@ DINUC_CATEGORIES = [[('A', 'A'), ('T', 'T')],
                     [('G', 'C')],
                     [('T', 'A')]]
 
-def start_R():
-    r_source("common.R")
-    r_source("dinucleotide.R")
+R = RInterface(["common.R", "dinucleotide.R"])
 
 ## Caclulates nucleotide and dinucleotide frequencies over the specified
-## segmentation
-def calc_nucleotide_frequencies(segmentation, genome,
+## annotations
+def calc_nucleotide_frequencies(annotations, genome,
                                 nuc_categories=NUC_CATEGORIES,
                                 dinuc_categories=DINUC_CATEGORIES,
                                 quick=False, verbose=True):
@@ -78,27 +78,33 @@ def calc_nucleotide_frequencies(segmentation, genome,
     nuc_counts = defaultdict(dict)
     dinuc_counts = defaultdict(dict)
 
-    labels = segmentation.labels
+    labels = annotations.labels
     for label_key in labels.iterkeys():
         nuc_counts[label_key] = zeros(len(nuc_categories)+1, dtype=numpy.long)
         dinuc_counts[label_key] = zeros(len(dinuc_categories)+1,
                                         dtype=numpy.long)
 
-    # Count (di)nucleotides over segmentation
+    # Count (di)nucleotides over annotations
     for chromosome in genome:
         chrom = chromosome.name
-        log("\t%s" % chrom, verbose)
 
         try:
-            segments = segmentation.chromosomes[chrom]
+            chrom_rows = annotations.chromosomes[chrom]
+            # Store entire chromosome's sequence as string in memory
+            # for speedup
+            warnings.simplefilter("ignore")
+            chrom_sequence = chromosome.seq[0:chromosome.end].tostring().upper()
+            warnings.resetwarnings()
         except KeyError:
             continue
 
-        for segment in segments:
-            seg_start, seg_end, seg_label = segment
+        log("  %s" % chrom, verbose)
+        for row in chrom_rows:
+            start = row['start']
+            end = row['end']
+            label = row['key']
 
-            sequence = chromosome.seq[seg_start:seg_end]
-            sequence = sequence.tostring().upper()
+            sequence = chrom_sequence[start:end]
 
             # XXXopt: could be optimized to use matrix operations
             # to determine number of occurances of each (di)nuc
@@ -113,17 +119,17 @@ def calc_nucleotide_frequencies(segmentation, genome,
                 nuc_cat = quick_nuc_categories[cur_nuc]
                 if nuc_cat == {}:  # Didn't find; put in last bin
                     #print >>sys.stderr, "Unknown nuc: %s" % (cur_nuc)
-                    nuc_cat = len(nuc_counts[seg_label]) - 1
+                    nuc_cat = len(nuc_counts[label]) - 1
 
-                nuc_counts[seg_label][nuc_cat] += 1
+                nuc_counts[label][nuc_cat] += 1
 
                 # Record dinucleotide (last and current)
                 if prev_nuc is not None:
                     dinuc_cat = quick_dinuc_categories[(prev_nuc, cur_nuc)]
                     if dinuc_cat == {}:  # Didn't find; put in last bin
-                        dinuc_cat = len(dinuc_counts[seg_label]) - 1
+                        dinuc_cat = len(dinuc_counts[label]) - 1
 
-                    dinuc_counts[seg_label][dinuc_cat] += 1
+                    dinuc_counts[label][dinuc_cat] += 1
 
         # Only look at first chromosome if quick
         if quick: break
@@ -159,31 +165,31 @@ def save_tab(labels, nuc_counts, dinuc_counts, dirpath,
 
 def save_plot(dirpath, clobber=False, verbose=True,
               mnemonic_file="", namebase=NAMEBASE):
-    start_R()
+    R.start(verbose=verbose)
 
     tabfilename = make_tabfilename(dirpath, NAMEBASE)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    r_plot("save.dinuc", dirpath, namebase, tabfilename,
-           mnemonic_file=mnemonic_file, clobber=clobber, verbose=verbose)
+    R.plot("save.dinuc", dirpath, namebase, tabfilename,
+           mnemonic_file=mnemonic_file, clobber=clobber)
 
-def save_html(dirpath, clobber=False, mnemonicfile=None):
+def save_html(dirpath, clobber=False, verbose=True, mnemonicfile=None):
     save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
-                  tables={"":NAMEBASE}, module=MODULE,
+                  tables={"":NAMEBASE}, module=MODULE, verbose=verbose,
                   mnemonicfile=mnemonicfile, title=HTML_TITLE)
 
 ## Package entry point
-def validate(bedfilename, genomedatadir, dirpath, clobber=False, quick=False,
+def validate(filename, genomedatadir, dirpath, clobber=False, quick=False,
              replot=False, noplot=False, mnemonic_file=None, verbose=True):
     setup_directory(dirpath)
     if not replot:
-        segmentation = Segmentation(bedfilename, verbose=verbose)
-        labels = segmentation.labels
+        annotations = Annotations(filename, verbose=verbose)
+        labels = annotations.labels
 
         with Genome(genomedatadir) as genome:
             nuc_counts, dinuc_counts = \
-                calc_nucleotide_frequencies(segmentation, genome,
+                calc_nucleotide_frequencies(annotations, genome,
                                             quick=quick, verbose=verbose)
 
         save_tab(labels, nuc_counts, dinuc_counts, dirpath,
@@ -193,12 +199,13 @@ def validate(bedfilename, genomedatadir, dirpath, clobber=False, quick=False,
         save_plot(dirpath, clobber=clobber, verbose=verbose,
                   mnemonic_file=mnemonic_file)
 
-    save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file)
+    save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file,
+              verbose=verbose)
 
 def parse_options(args):
     from optparse import OptionGroup, OptionParser
 
-    usage = "%prog [OPTIONS] SEGMENTATION GENOMEDATAFILE"
+    usage = "%prog [OPTIONS] ANNOTATIONS GENOMEDATAFILE"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
 
@@ -243,16 +250,12 @@ def parse_options(args):
 ## Command-line entry point
 def main(args=sys.argv[1:]):
     (options, args) = parse_options(args)
-    bedfilename = args[0]
+    filename = args[0]
     genomedatadir = args[1]
 
-    kwargs = {"clobber": options.clobber,
-              "verbose": options.verbose,
-              "quick": options.quick,
-              "replot": options.replot,
-              "noplot": options.noplot,
-              "mnemonic_file": options.mnemonic_file}
-    validate(bedfilename, genomedatadir, options.outdir, **kwargs)
+    kwargs = dict(options.__dict__)
+    outdir = kwargs.pop("outdir")
+    validate(filename, genomedatadir, outdir, **kwargs)
 
 if __name__ == "__main__":
     sys.exit(main())
