@@ -1,24 +1,26 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 273 $"
+__version__ = "$Revision: 321 $"
 
 """
 Evaluates the overlap between two BED files, based upon the spec at:
 http://encodewiki.ucsc.edu/EncodeDCC/index.php/Overlap_analysis_tool_specification
-
-Author: Orion Buske <stasis@uw.edu>
-Date:   August 18, 2009
 """
 
+# Author: Orion Buske <stasis@uw.edu>
+# Date:   August 18, 2009
+
+
 import os
 import sys
 
 from collections import defaultdict
 from numpy import bincount, cast, iinfo, invert, logical_or, zeros
 
-from . import Annotations, log, Segmentation
-from .common import check_clobber, die, get_ordered_labels, make_tabfilename, r_plot, r_source, setup_directory, SUFFIX_GZ, tab_saver
+from . import Annotations, log, open_transcript, Segmentation, RInterface
+from .common import check_clobber, die, get_ordered_labels, make_tabfilename, \
+     setup_directory, SUFFIX_GZ, tab_saver
 from .html import save_html_div
 
 # A package-unique, descriptive module name used for filenames, etc
@@ -50,9 +52,7 @@ PNG_SIZE_PER_PANEL = 400  # px
 SIGNIFICANCE_PNG_SIZE = 600  # px
 HEATMAP_PNG_SIZE = 600 # px
 
-def start_R():
-    r_source("common.R")
-    r_source("overlap.R")
+R = RInterface(["common.R", "overlap.R"])
 
 def calc_overlap(subseg, qryseg, quick=False, clobber=False, mode=MODE_DEFAULT,
                  print_segments=False, dirpath=None, verbose=True,
@@ -257,34 +257,37 @@ def save_tab(dirpath, row_labels, col_labels, counts, totals, nones, mode,
 
 def save_plot(dirpath, namebase=NAMEBASE, clobber=False, verbose=True,
               row_mnemonic_file=None, col_mnemonic_file=None,
-              cluster=False):
-    start_R()
+              cluster=False, max_contrast=False, transcriptfile=None):
+    R.start(transcriptfile, verbose=verbose)
 
     tabfilename = make_tabfilename(dirpath, namebase)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    r_plot("save.overlap.heatmap", dirpath, namebase, tabfilename,
+    R.plot("save.overlap.heatmap", dirpath, namebase, tabfilename,
            mnemonic_file=row_mnemonic_file,
            col_mnemonic_file=col_mnemonic_file,
-           clobber=clobber, cluster=cluster, verbose=verbose)
+           clobber=clobber, cluster=cluster,
+           max_contrast=max_contrast)
 
 def save_performance_plot(dirpath, namebase=PERFORMANCE_NAMEBASE,
                           clobber=False, verbose=True,
-                          row_mnemonic_file=None, col_mnemonic_file=None):
-    start_R()
+                          row_mnemonic_file=None, col_mnemonic_file=None,
+                          transcriptfile=None):
+    R.start(transcriptfile, verbose=verbose)
 
     tabfilename = make_tabfilename(dirpath, NAMEBASE)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    r_plot("save.overlap.performance", dirpath, namebase, tabfilename,
+    R.plot("save.overlap.performance", dirpath, namebase, tabfilename,
            mnemonic_file=row_mnemonic_file,
            col_mnemonic_file=col_mnemonic_file,
-           clobber=clobber, verbose=verbose)
+           clobber=clobber)
+
 
 def save_html(dirpath, bedfilename, featurefilename, mode,
-              mnemonicfile=None, clobber=False):
+              mnemonicfile=None, clobber=False, verbose=True):
     bedfilename = os.path.basename(bedfilename)
     featurebasename = os.path.basename(featurefilename)
     extra_namebases = {"performance": PERFORMANCE_NAMEBASE}
@@ -294,7 +297,7 @@ def save_html(dirpath, bedfilename, featurefilename, mode,
     significance = ""
     save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                   title=title, tables={"":(PERFORMANCE_NAMEBASE, "exact")},
-                  mnemonicfile=mnemonicfile,
+                  mnemonicfile=mnemonicfile, verbose=verbose,
                   extra_namebases=extra_namebases,
                   module=MODULE, by=mode, significance=significance,
                   bedfilename=bedfilename, featurefilename=featurebasename)
@@ -317,8 +320,8 @@ def overlap(bedfilename, featurefilename, dirpath, regionfilename=None,
             region_fraction=REGION_FRACTION_DEFAULT,
             subregion_fraction=SUBREGION_FRACTION_DEFAULT, min_overlap=1,
             mnemonic_filename=None, feature_mnemonic_filename=None,
-            replot=False, noplot=False, cluster=False, verbose=True):
-
+            replot=False, noplot=False, cluster=False, max_contrast=False,
+            verbose=True):
     if not replot:
         setup_directory(dirpath)
 
@@ -342,17 +345,24 @@ def overlap(bedfilename, featurefilename, dirpath, regionfilename=None,
                  clobber=clobber, verbose=verbose)
 
     if not noplot:
-        save_performance_plot(dirpath, clobber=clobber, verbose=verbose,
-                              row_mnemonic_file=mnemonic_filename,
-                              col_mnemonic_file=feature_mnemonic_filename)
-        save_plot(dirpath, clobber=clobber, cluster=cluster, verbose=verbose,
-                  row_mnemonic_file=mnemonic_filename,
-                  col_mnemonic_file=feature_mnemonic_filename)
+        with open_transcript(dirpath, MODULE, True) as transcriptfile:
+            # Performance plot is only valid for bases
+            if mode == "bases":
+                save_performance_plot(\
+                    dirpath, clobber=clobber, verbose=verbose,
+                    row_mnemonic_file=mnemonic_filename,
+                    col_mnemonic_file=feature_mnemonic_filename,
+                    transcriptfile=transcriptfile)
+            else:
+                log("Not in base-mode, so skipping performance plot", verbose)
+
+            save_plot(dirpath, clobber=clobber, cluster=cluster,
+                      verbose=verbose, row_mnemonic_file=mnemonic_filename,
+                      col_mnemonic_file=feature_mnemonic_filename,
+                      max_contrast=max_contrast, transcriptfile=transcriptfile)
 
-    log("Saving HTML div...", verbose)
     save_html(dirpath, bedfilename, featurefilename, mode=mode,
-              mnemonicfile=mnemonic_filename, clobber=clobber)
-    log(" done", verbose)
+              mnemonicfile=mnemonic_filename, clobber=clobber, verbose=verbose)
 
 def parse_options(args):
     from optparse import OptionParser, OptionGroup
@@ -372,34 +382,33 @@ Overlap_analysis_tool_specification"
                           description=description)
 
     group = OptionGroup(parser, "Flags")
-    group.add_option("--clobber", action="store_true",
-                     dest="clobber", default=False,
+    group.add_option("--clobber", action="store_true", default=False,
                      help="Overwrite existing output files if the specified"
                      " directory already exists.")
     group.add_option("-q", "--quiet", action="store_false",
                      dest="verbose", default=True,
                      help="Do not print diagnostic messages.")
-    group.add_option("--quick", action="store_true",
-                     dest="quick", default=False,
+    group.add_option("--quick", action="store_true", default=False,
                      help="Compute values only for one chromosome.")
-    group.add_option("--replot", action="store_true",
-                     dest="replot", default=False,
+    group.add_option("--replot", action="store_true", default=False,
                      help="Load data from output tab files and"
                      " regenerate plots instead of recomputing data")
-    group.add_option("--noplot", action="store_true",
-                     dest="noplot", default=False,
+    group.add_option("--noplot", action="store_true", default=False,
                      help="Do not generate plots")
-    group.add_option("--cluster", action="store_true",
-                     dest="cluster", default=False,
-                     help="Cluster rows and columns in heatmap plot")
+    group.add_option("--cluster", action="store_true", default=False,
+                     help="Cluster rows and columns in heat map plot")
     group.add_option("-p", "--print-segments", action="store_true",
-                     dest="print_segments", default=False,
+                     default=False,
                      help=("For each group"
                      " in the SEGMENTATION, a separate output file will be"
                      " created that contains a list of all the segments that"
                      " the group was found to overlap with. Output files"
                      " are named %s.X.txt, where X is the name"
-                     " of the SEGMENTATION group.") % OVERLAPPING_SEGMENTS_NAMEBASE)
+                     " of the SEGMENTATION group.")
+                     % OVERLAPPING_SEGMENTS_NAMEBASE)
+    group.add_option("--max-contrast", action="store_true",
+                     help="Saturate color range instead of having it go from"
+                     " 0 to 1")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Parameters")
@@ -463,6 +472,7 @@ def main(args=sys.argv[1:]):
               "print_segments": options.print_segments,
               "mode": options.mode,
               "min_overlap": options.min_overlap,
+              "max_contrast": options.max_contrast,
               "mnemonic_filename": options.mnemonic_filename,
               "feature_mnemonic_filename": options.feature_mnemonic_filename,
               "verbose": options.verbose}
